/*
 * Copyright (c) 2010-2011 by inovat, innovative systeme - verkehr - tunnel - technik, Dipl.-Ing. H. C. Kniss
 *
 * This file is part of de.inovat.dua.pufferlzzs.Auftrag
 *
 * de.inovat.dua.pufferlzzs.Auftrag is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * de.inovat.dua.pufferlzzs.Auftrag is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * <p>
 * You should have received a copy of the GNU General Public License
 * along with de.inovat.dua.pufferlzzs.Auftrag; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Contact Information:
 * inovat, Dipl.-Ing. H. C. Kniss
 * Koelner Strasse 30
 * D-50859 Koeln
 * +49 (0)2234 4301 800
 * info@invat.de
 * www.inovat.de
 */



package de.inovat.dua.pufferlzzs;

//~ JDK IMPORTE ===============================================================

import java.text.SimpleDateFormat;
import java.util.Date;

//~ KLASSEN ===================================================================

/**
 * Enthält alle Informationen für einen Pufferabrufauftrag.
 *
 * @author inovat, innovative systeme - verkehr - tunnel - technik
 * @author Dipl.-Ing. Hans Christian Kniß (HCK)
 * @version $Revision$ / $Date$ / ($Author$)
 */
public class Auftrag {

    /** IntervallBeginn */
    private long _intervallBeginn;

    /** IntervallEnde: Intervallbeginn + Intervalllänge (ausschließlich!, also der nächste Intervallbeginn */
    private long _intervallEnde;

    /** VewDeLve255 - Verwaltungsobjekt, welches den Auftrag ausführen soll */
    private VewDeLve255 _vewDeLve255;

    /** Zeitpunkt der geplanten Ausführung für diesen Auftrag in Millisekunden seit 1.1.1970 */
    private long _ausfuehrungsZeitPunkt = 0;

    /** Datumsformatierung für Ausgabe toString() */
    SimpleDateFormat _datumsFormatierer = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss,SSS");

    //~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ==============

    /**
     * Konstruktor, um einen Folgeauftrag auf Basis eines bestehenden Auftrags zu berechnen
     *
     * @param auftrag Aktueller Auftrag, für den der Folgeauftrag bestimmt werden soll
     */
    public Auftrag(Auftrag auftrag) {
        long intervallDauer    = auftrag.getIntervallEnde() - auftrag.getIntervallBeginn();
        long minimaleWarteZeit = auftrag.getAusfuehrungsZeitPunkt() - auftrag.getIntervallEnde();

        this._intervallBeginn       = auftrag.getIntervallBeginn() + intervallDauer;
        this._intervallEnde         = auftrag.getIntervallEnde() + intervallDauer;
        this._vewDeLve255           = auftrag.getVewDeLve255();
        this._ausfuehrungsZeitPunkt = this._intervallEnde + minimaleWarteZeit;
    }

    /**
     * Konstruktor, um den initialen Erstauftrag für ein Verwaltungsobjekt und das erste zu berechnende Intervall
     * anzulegen
     *
     * @param ausfuehrungsZeitpunkt Zeitpunkt, zu dem Auftrag frühestens ausgeführt werden soll.
     * @param intervallBeginn       Intervallbeginn (EINSCHLIESSLICH!!) in Millisekunden seit 1.1.1970
     * @param intervallEnde         Intervallende (AZSSCHLIESSLICH!!) in Millisekunden seit 1.1.1970
     * @param vewDeLve255           Verwaltungsobjekt, welches den Auftrag letztendlich ausführen soll
     */
    public Auftrag(long ausfuehrungsZeitpunkt, long intervallBeginn, long intervallEnde, VewDeLve255 vewDeLve255) {
        _ausfuehrungsZeitPunkt = ausfuehrungsZeitpunkt;
        _intervallBeginn       = intervallBeginn;
        _intervallEnde         = intervallEnde;
        _vewDeLve255           = vewDeLve255;
    }

    //~ METHODEN ==============================================================

    /**
     * Führt den Auftrag aus, indem am zugeordneten Verwaltungsobjekt die Methode {@link
     * VewDeLve255#ausfuehren(de.inovat.dua.pufferlzzs.Auftrag)} aufgerufen wir.
     *
     * @return Meldung mit dem Ergebnis der Ausführung des Auftrags.
     */
    public String ausfuehren() {
        return _vewDeLve255.ausfuehren(this);
    }

    //~ GET METHODEN ==========================================================

    /**
     * Liefert den Zeitpunkt der geplanten Ausführung für diesen Auftrag in Millisekunden seit 1.1.1970
     *
     * @return Zeitpunkt der geplanten Ausführung für diesen Auftrag in Millisekunden seit 1.1.1970
     */
    public long getAusfuehrungsZeitPunkt() {
        return _ausfuehrungsZeitPunkt;
    }

    /**
     * Liefert den Zeitpunkt des Intervallbeginns (EINSCHLIESSLICH!!) in Millisekunden seit 1.1.1970
     *
     * @return Zeitpunkt des Intervallbeginns (EINSCHLIESSLICH!!) in Millisekunden seit 1.1.1970
     */
    public long getIntervallBeginn() {
        return _intervallBeginn;
    }

    /**
     * Liefert den Zeitpunkt des Intervallendes (AUSSCHLIESSLICH!!) in Millisekunden seit 1.1.1970
     *
     * @return Zeitpunkt des Intervallendes (AUSSCHLIESSLICH!!) in Millisekunden seit 1.1.1970
     */
    public long getIntervallEnde() {
        return _intervallEnde;
    }

    /**
     * Liefert das Verwaltungsobjekt, welches den Auftrag ausführen soll.
     *
     * @return Verwaltungsobjekt, welches den Auftrag ausführen soll.
     */
    public VewDeLve255 getVewDeLve255() {
        return _vewDeLve255;
    }

    //~ METHODEN ==============================================================

    @Override

    /**
     * Gibt die Textrepräsentation für einen Auftrag aus.
     */
    public String toString() {
        String ausfuehrung = "DauerAuftrag";
        String art         = "PufferAbruf";

        return String.format("Auftrag (%-12s- %-15s) Ausführung am/um %s: Intervall %s - %s fuer %s", art, ausfuehrung, _datumsFormatierer.format(new Date(_ausfuehrungsZeitPunkt)), _datumsFormatierer.format(new Date(_intervallBeginn)), _datumsFormatierer.format(new Date(_intervallEnde)), _vewDeLve255.getPid());
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 08.12.10
